﻿// ConnectTheElements by Charles Petzold, February 2010
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace ConnectTheElements
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();
            LayoutUpdated += OnLayoutUpdated;
        }

        void OnLayoutUpdated(object sender, EventArgs args)
        {
            WrapEllipseAroundElement(btnA, ellipseGeometryA);
            WrapEllipseAroundElement(btnB, ellipseGeometryB);

            double angle = Math.Atan2(ellipseGeometryB.Center.Y - ellipseGeometryA.Center.Y,
                                      ellipseGeometryB.Center.X - ellipseGeometryA.Center.X);

            line.StartPoint = PointOnEllipse(ellipseGeometryA, angle);
            line.EndPoint = PointOnEllipse(ellipseGeometryB, -angle);
        }

        void WrapEllipseAroundElement(FrameworkElement element, EllipseGeometry ellipseGeo)
        {
            Point center = new Point(element.ActualWidth / 2, element.ActualHeight / 2);
            ellipseGeo.Center = element.TransformToVisual(this).Transform(center);
            ellipseGeo.RadiusX = 1.414 * center.X;
            ellipseGeo.RadiusY = 1.414 * center.Y;
        }

        Point PointOnEllipse(EllipseGeometry ellipseGeo, double angle)
        {
            return PointOnEllipse(ellipseGeo.Center, ellipseGeo.RadiusX, ellipseGeo.RadiusY, angle);
        }

        Point PointOnEllipse(Point center, double radiusX, double radiusY, double angle)
        {
            Point pt = new Point();
            pt.X = radiusX * radiusY / Math.Sqrt(radiusY * radiusY + 
                                                 radiusX * radiusX * Math.Pow(Math.Tan(angle), 2));

            if (angle > Math.PI / 2 || angle < -Math.PI / 2)
                pt.X *= -1;

            pt.Y = radiusY * Math.Sqrt(1 - Math.Pow(pt.X / radiusX, 2));

            if (angle < 0)
            {
                pt.X *= -1;
                pt.Y *= -1;
            }

            pt.X += center.X;
            pt.Y += center.Y;

            return pt;
        }
    }
}
