﻿// SimpleUniformGrid.cs by Charles Petzold, February 2010
using System;
using System.Windows;
using System.Windows.Controls;

namespace ConnectTheElements
{
    public class SimpleUniformGrid : Panel
    {
        int rows, cols;
        Size maxChildSize;

        protected override Size MeasureOverride(Size availableSize)
        {
            maxChildSize = new Size();

            if (Children.Count == 0)
                return new Size();

            foreach (UIElement child in Children)
            {
                child.Measure(new Size(Double.PositiveInfinity, 
                                       Double.PositiveInfinity));
                Size childSize = child.DesiredSize;
                maxChildSize.Width = Math.Max(maxChildSize.Width, childSize.Width);
                maxChildSize.Height = Math.Max(maxChildSize.Height, childSize.Height);
            }

            if ((Double.IsPositiveInfinity(availableSize.Width) &&
                 Double.IsPositiveInfinity(availableSize.Height)) ||
                (maxChildSize.Width == 0 && maxChildSize.Height == 0))
            {
                rows = (int)(Math.Sqrt(Children.Count) + 0.5);
                cols = (Children.Count + rows - 1) / rows;
            }
            else if ((Double.IsPositiveInfinity(availableSize.Width) && 
                      maxChildSize.Height != 0) ||
                     maxChildSize.Width == 0)
            {
                rows = (int)Math.Max(1, availableSize.Height / maxChildSize.Height);
                cols = (Children.Count + rows - 1) / rows;
            }
            else if ((Double.IsPositiveInfinity(availableSize.Height) && 
                      maxChildSize.Width != 0) ||
                     maxChildSize.Height == 0)
            {
                cols = (int)Math.Max(1, availableSize.Width / maxChildSize.Width);
                rows = (Children.Count + cols - 1) / cols;
            }
            else // non-infinite availableSize, non-zero maxChildSize
            {
                int possibleCols = (int)(availableSize.Width / maxChildSize.Width);
                int possibleRows = (int)(availableSize.Height / maxChildSize.Height);
                double possibleCells = possibleCols * possibleRows;
                double scale = Math.Sqrt(Children.Count / possibleCells);
                cols = (int)Math.Max(1, scale * possibleCols);
                rows = (Children.Count + cols - 1) / cols;
            }

            return new Size(cols * maxChildSize.Width, 
                            rows * maxChildSize.Height);
        }

        protected override Size ArrangeOverride(Size finalSize)
        {
            Size childSize = new Size(finalSize.Width / cols, finalSize.Height / rows);

            for (int index = 0; index < Children.Count; index++)
            {
                UIElement child = Children[index];
                double x = childSize.Width * (index % cols);
                double y = childSize.Height * (index / cols);
                child.Arrange(new Rect(new Point(x, y), childSize));
            }
            
            return base.ArrangeOverride(finalSize);
        }
    }
}
