﻿// ItemsControlPopouts by Charles Petzold, February 2010
using System;
using System.Windows;
using System.Windows.Controls;

namespace ItemsControlPopouts
{
    public partial class MainPage : UserControl
    {
        ProduceItemsPresenter presenter;
        ItemContainerGenerator itemsControlGenerator;

        public MainPage()
        {
            InitializeComponent();

            presenter = new ProduceItemsPresenter();
            LayoutRoot.DataContext = presenter;

            itemsControlGenerator = itemsControl.ItemContainerGenerator;
            LayoutUpdated += OnMainPageLayoutUpdated;
        }

        void OnMainPageLayoutUpdated(object sender, EventArgs args)
        {
            if (presenter == null || presenter.ProduceItems == null)
                return;

            Point scrollViewerTop = 
                scrollViewer.TransformToVisual(this).Transform(new Point());
            Point scrollViewerBot = 
                scrollViewer.TransformToVisual(this).Transform(new Point(0, scrollViewer.ActualHeight));

            foreach (ProduceItem produceItem in presenter.ProduceItems.Items)
            {
                if (String.IsNullOrEmpty(produceItem.Message))
                    continue;

                FrameworkElement container =
                    itemsControlGenerator.ContainerFromItem(produceItem) as FrameworkElement;

                if (container == null)
                    continue;

                Point elPoint = new Point(container.ActualWidth, container.ActualHeight / 2);
                Point canvasPoint = container.TransformToVisual(this).Transform(elPoint);
                string message = produceItem.Message;
                PopOut popout = null;

                // Check if the TextBlock has already been created
                foreach (UIElement child in canvas.Children)
                {
                    if (child is PopOut && (child as PopOut).Content as string == message)
                    {
                        popout = child as PopOut;
                        break;
                    }
                }

                // If not, create it
                if (popout == null)
                {
                    popout = new PopOut();
                    popout.Content = message;
                    canvas.Children.Add(popout);
                }

                Canvas.SetLeft(popout, canvasPoint.X);
                Canvas.SetTop(popout, canvasPoint.Y - popout.ActualHeight / 2);

                popout.Visibility = 
                    (canvasPoint.Y > scrollViewerTop.Y &&
                     canvasPoint.Y < scrollViewerBot.Y) ? Visibility.Visible : Visibility.Collapsed;
            }
        }
    }
}
