﻿// TriangleAngles by Charles Petzold, February 2010
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Media;

namespace TriangleAngles
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();
            LayoutUpdated += OnMainPageLayoutUpdated;
        }

        void OnMainPageLayoutUpdated(object sender, EventArgs args)
        {
            Thumb vertThumb = FindChild<Thumb>(vertSlider, true);
            Thumb horzThumb = FindChild<Thumb>(horzSlider, true);

            if (vertThumb != null && horzThumb != null)
            {
                // Find the centers of the slider thumbs relative to the Grid (and canvas)
                Point horzThumbCenter = new Point(horzThumb.ActualWidth / 2,
                                                  horzThumb.ActualHeight / 2);
                Point horzPoint =
                    horzThumb.TransformToVisual(LayoutRoot).Transform(horzThumbCenter);

                Point vertThumbCenter = new Point(vertThumb.ActualWidth / 2,
                                                  vertThumb.ActualHeight / 2);
                Point vertPoint =
                    vertThumb.TransformToVisual(LayoutRoot).Transform(vertThumbCenter);

                // Set the three points in the Polygon
                triangle.Points[0] = new Point(vertPoint.X, horzPoint.Y);
                triangle.Points[1] = vertPoint;
                triangle.Points[2] = horzPoint;

                // Set the degrees near the horizontal slider
                double horzDegrees = 180 * Math.Atan2(horzPoint.Y - vertPoint.Y, 
                                                      horzPoint.X - vertPoint.X) / Math.PI;
                horzAngle.Text = String.Format("{0:F1}°", horzDegrees);
                Canvas.SetLeft(horzAngle, horzPoint.X - 
                    horzAngle.ActualHeight / Math.Tan(Math.PI * horzDegrees / 180) - 
                    horzAngle.ActualWidth);
                Canvas.SetTop(horzAngle, horzPoint.Y - horzAngle.ActualHeight);

                // Set the degrees near the vertical slider
                double vertDegrees = 90 - horzDegrees;
                vertAngle.Text = String.Format("{0:F1}°", vertDegrees);
                Canvas.SetLeft(vertAngle, vertPoint.X + 2);
                Canvas.SetTop(vertAngle, vertPoint.Y + 
                    (vertAngle.ActualWidth + 2) / Math.Tan(Math.PI * vertDegrees / 180));

                // Set the text on the diagonal
                Canvas.SetLeft(diagLabel, (horzPoint.X + vertPoint.X) / 2);
                Canvas.SetTop(diagLabel, (vertPoint.Y + horzPoint.Y) / 2 - diagLabel.ActualHeight);

                // Set the text on the horizontal axis
                horzLabel.Text = String.Format("Sin({0:F1}°) = Cos({1:F1}°) = {2:F3}", 
                                               vertDegrees, horzDegrees, 
                                               Math.Sin(Math.PI * vertDegrees / 180));
                Canvas.SetLeft(horzLabel, (horzPoint.X + vertPoint.X - horzLabel.ActualWidth) / 2);
                Canvas.SetTop(horzLabel, horzPoint.Y + horzThumb.ActualHeight / 2);

                // Set the text on the vertical axis
                vertLabel.Text = String.Format("Sin({0:F1}°) = Cos({1:F1}°) = {2:F3}", 
                                               horzDegrees, vertDegrees, 
                                               Math.Sin(Math.PI * horzDegrees / 180));
                Point midPoint = new Point(vertPoint.X, (horzPoint.Y + vertPoint.Y) / 2);
                Canvas.SetLeft(vertLabel, midPoint.X - vertLabel.ActualHeight - vertThumb.ActualWidth / 2);
                Canvas.SetTop(vertLabel, midPoint.Y + vertLabel.ActualWidth / 2); 
             }
        }

        T FindChild<T>(DependencyObject parent, bool mustBeVisible) where T : DependencyObject
        {
            if (mustBeVisible == true && parent is UIElement &&
                (parent as UIElement).Visibility != Visibility.Visible)
            {
                return null;
            }

            if (parent is T)
                return parent as T;

            int numChildren = VisualTreeHelper.GetChildrenCount(parent);

            for (int childIndex = 0; childIndex < numChildren; childIndex++)
            {
                T child = FindChild<T>(VisualTreeHelper.GetChild(parent, childIndex), mustBeVisible);

                if (child != null)
                    return child;
            }
            return null;
        }
    }
}
